import {
  OnTransactionHandler,
  OnRpcRequestHandler,
} from '@metamask/snaps-types';
import { panel, heading, text, copyable, divider } from '@metamask/snaps-ui';
import { Network } from 'alchemy-sdk';
import web3 from 'web3';

export const onTransaction: OnTransactionHandler = async ({
  transaction,
  chainId,
  transactionOrigin,
}) => {
  let network;
  switch (chainId) {
    case 'eip155:1':
      network = Network.ETH_MAINNET;
      break;
    case 'eip155:5':
      network = Network.ETH_GOERLI;
      break;
    case 'eip155:11155111':
      network = Network.ETH_SEPOLIA;
      break;
    case 'eip155:137':
      network = Network.POLYGONZKEVM_MAINNET;
      break;
    case 'eip155:80001':
      network = Network.POLYGONZKEVM_TESTNET;
      break;
    case 'eip155:421613':
      network = Network.ARB_GOERLI;
      break;
    case 'eip155:42161':
      network = Network.ARB_MAINNET;
      break;
    default:
      network = Network.ETH_MAINNET;
      break;
  }
  const persistedData = await snap.request({
    method: 'snap_manageState',
    params: { operation: 'get' },
  });
  let api_key_alchemy: any = persistedData?.api_key_alchemy;
  const responseAlchemy = await fetch(
    `https://${network}.g.alchemy.com/v2/${api_key_alchemy}`,
    {
      method: 'POST',
      headers: {
        accept: 'application/json',
        'content-type': 'application/json',
      },
      body: JSON.stringify({
        id: 1,
        jsonrpc: '2.0',
        method: 'alchemy_simulateAssetChanges',
        params: [
          {
            from: transaction.from,
            to: transaction.to,
            value: transaction.value,
            data: transaction.data,
            gas: transaction.gas,
            maxFeePerGas: transaction.maxFeePerGas,
            maxPriorityFeePerGas: transaction.maxPriorityFeePerGas,
          },
        ],
      }),
    },
  );
  let dataAlchemy = await responseAlchemy.json();
  if (dataAlchemy.error) {
    return {
      content: panel([
        heading(`Error:`),
        divider(),
        text(`chainId = ${chainId}`),
        text(`transactionOrigin = ${transactionOrigin}`),
        divider(),
        copyable(`${dataAlchemy?.error?.message}`),
      ]),
    };
  }
  let messageGPT;
  if (dataAlchemy.result.changes.length < 2) {
    messageGPT = `A user is about to sign a blockchain transaction with their Metamask wallet. Can you explain with one or 2 sentences in a simple to understand language, 
    what will happen as a result of this transaction if the data payload looks like this: 
    '{'changes”: [{'changeType': '${
      dataAlchemy.result.changes[0]
        ? dataAlchemy.result.changes[0].changeType
        : ''
    },
    'from': '${transaction.from},
    'to': '${transaction.to},
    'contractAddress”: ${
      dataAlchemy.result.changes[0]
        ? dataAlchemy.result.changes[0].contractAddress
        : ''
    },
    'tokenId': ${
      dataAlchemy.result.changes[0] ? dataAlchemy.result.changes[0].tokenId : ''
    },
    'symbol': '${
      dataAlchemy.result.changes[0] ? dataAlchemy.result.changes[0].symbol : ''
    }',
    'name': '${
      dataAlchemy.result.changes[0] ? dataAlchemy.result.changes[0].name : ''
    }',
    'amount': '${
      dataAlchemy.result.changes[0] ? dataAlchemy.result.changes[0].amount : ''
    }'}],
    'error': ${dataAlchemy.result.error}}' 
    Also, specify if there are any risks for the user associated with this transaction`;
  } else {
    messageGPT = `A user is about to sign a blockchain swap with their Metamask wallet. Can you explain with one or 2 sentences in a simple to understand language, 
  what will happen as a result of this swap if the data payload looks like this: 
  transaction: ${JSON.stringify(transaction)},
  chainId: ${chainId},
  transactionOrigin: ${transactionOrigin},
  Also, specify if there are any risks for the user associated with this transaction (yes or no)`;
  }
  let api_key_chat_gpt: any = persistedData?.api_key_gpt;
  const responseGPT = await fetch(
    `https://api.openai.com/v1/chat/completions`,
    {
      method: 'POST',
      headers: {
        'content-type': 'application/json',
        Authorization: api_key_chat_gpt,
      },
      body: JSON.stringify({
        model: 'gpt-3.5-turbo-0613',
        messages: [{ role: 'user', content: messageGPT }],
        temperature: 0.6,
        max_tokens: 76,
        top_p: 0.4,
        frequency_penalty: 2.0,
        presence_penalty: 2.0,
        stop: ['You:'],
      }),
    },
  );
  let dataGPT = await responseGPT.json();
  let TextGPT = dataGPT.error
    ? JSON.stringify(dataGPT.error)
    : dataGPT.choices[0].message.content;
  return {
    content: panel([
      heading(`Transaction Insights`),
      divider(),
      text(TextGPT ? TextGPT : ''),
      dataAlchemy.result.changes[0]
        ? text(`**Type:** ` + dataAlchemy.result.changes[0].changeType)
        : text(''),
      text(`**From:** ` + transaction.from),
      text(`**To:** ` + transaction.to),
      dataAlchemy.result.changes[0]
        ? text(`**Symbol:** ` + dataAlchemy.result.changes[0].symbol)
        : text(''),
      dataAlchemy.result.changes[0]
        ? text(`**Amount:** ` + dataAlchemy.result.changes[0].amount)
        : text(''),
      text(
        `**gasUsed:** ` +
          web3.utils.hexToNumber(
            dataAlchemy.result.gasUsed
              ? dataAlchemy.result.gasUsed
              : transaction.gas,
          ),
      ),
    ]),
  };
};
export const onRpcRequest: OnRpcRequestHandler = async ({ request }) => {
  switch (request.method) {
    case 'setApiKeys':
      const { api_key_chat_gpt } = request.params as {
        api_key_chat_gpt: string;
      };
      const { api_key_alchemy } = request.params as {
        api_key_alchemy: string;
      };

      return snap.request({
        method: 'snap_manageState',
        params: {
          operation: 'update',
          newState: {
            api_key_gpt: api_key_chat_gpt,
            api_key_alchemy: api_key_alchemy,
          },
        },
      });
    default:
      throw new Error('Method not found.');
  }
};
